@@ -229,6 +229,10 @@ make_oci_process_user (const char *userspec)
 
 #define cleanup_process_schema __attribute__ ((cleanup (cleanup_process_schemap)))
 
+#ifdef SHARED_LIBCRUN
+void __attribute__ ((weak)) free_runtime_spec_schema_config_schema_process (runtime_spec_schema_config_schema_process *ptr);
+#endif
+
 static inline void
 cleanup_process_schemap (runtime_spec_schema_config_schema_process **p)
 {
@@ -291,10 +295,10 @@ crun_command_exec (struct crun_global_arguments *global_args, int argc, char **a
       process->user = make_oci_process_user (exec_options.user);
 
       if (exec_options.process_label != NULL)
-        process->selinux_label = exec_options.process_label;
+        process->selinux_label = xstrdup (exec_options.process_label);
 
       if (exec_options.apparmor != NULL)
-        process->apparmor_profile = exec_options.apparmor;
+        process->apparmor_profile = xstrdup (exec_options.apparmor);
 
       if (exec_options.cap_size > 0)
         {
@@ -304,8 +308,8 @@ crun_command_exec (struct crun_global_arguments *global_args, int argc, char **a
           capabilities->effective = exec_options.cap;
           capabilities->effective_len = exec_options.cap_size;
 
-          capabilities->inheritable = dup_array (exec_options.cap, exec_options.cap_size);
-          capabilities->inheritable_len = exec_options.cap_size;
+          capabilities->inheritable = NULL;
+          capabilities->inheritable_len = 0;
 
           capabilities->bounding = dup_array (exec_options.cap, exec_options.cap_size);
           capabilities->bounding_len = exec_options.cap_size;
