@@ -30,6 +30,7 @@
 #include "utils.h"
 #include "status.h"
 #include <string.h>
+#include <sched.h>
 #include <sys/types.h>
 #include <signal.h>
 #include <sys/vfs.h>
@@ -56,6 +57,7 @@ libcrun_destroy_cgroup_disabled (struct libcrun_cgroup_status *cgroup_status arg
 }
 
 struct libcrun_cgroup_manager cgroup_manager_disabled = {
+  .precreate_cgroup = NULL,
   .create_cgroup = libcrun_cgroup_enter_disabled,
   .destroy_cgroup = libcrun_destroy_cgroup_disabled,
 };
@@ -79,7 +81,7 @@ get_cgroup_manager (int manager, struct libcrun_cgroup_manager **out, libcrun_er
     }
 
   *out = NULL;
-  return crun_make_error (err, EINVAL, "unknown cgroup manager specified %d", manager);
+  return crun_make_error (err, EINVAL, "unknown cgroup manager specified `%d`", manager);
 }
 
 static const char *
@@ -252,6 +254,32 @@ can_ignore_cgroup_enter_errors (struct libcrun_cgroup_args *args, int cgroup_mod
   return 1;
 }
 
+int
+libcrun_cgroup_preenter (struct libcrun_cgroup_args *args, int *dirfd, libcrun_error_t *err)
+{
+  struct libcrun_cgroup_manager *cgroup_manager;
+  int cgroup_mode;
+  int ret;
+
+  *dirfd = -1;
+
+  cgroup_mode = libcrun_get_cgroup_mode (err);
+  if (UNLIKELY (cgroup_mode < 0))
+    return cgroup_mode;
+
+  if (cgroup_mode != CGROUP_MODE_UNIFIED)
+    return 0;
+
+  ret = get_cgroup_manager (args->manager, &cgroup_manager, err);
+  if (UNLIKELY (ret < 0))
+    return ret;
+
+  if (cgroup_manager->precreate_cgroup == NULL)
+    return 0;
+
+  return cgroup_manager->precreate_cgroup (args, dirfd, err);
+}
+
 int
 libcrun_cgroup_enter (struct libcrun_cgroup_args *args, struct libcrun_cgroup_status **out, libcrun_error_t *err)
 {
@@ -277,7 +305,7 @@ libcrun_cgroup_enter (struct libcrun_cgroup_args *args, struct libcrun_cgroup_st
     {
       ret = TEMP_FAILURE_RETRY (kill (args->pid, SIGSTOP));
       if (UNLIKELY (ret < 0))
-        return crun_make_error (err, errno, "cannot stop container process '%d' with SIGSTOP", args->pid);
+        return crun_make_error (err, errno, "cannot stop container process `%d` with SIGSTOP", args->pid);
 
       /* Send SIGCONT as soon as the function exits.  */
       sigcont_cleanup = args->pid;
@@ -352,6 +380,21 @@ libcrun_cgroup_enter (struct libcrun_cgroup_args *args, struct libcrun_cgroup_st
             return ret;
         }
     }
+  /* Reset the inherited cpu affinity. Old kernels do that automatically, but
+     new kernels remember the affinity that was set before the cgroup move.
+     This is undesirable, because it inherits the systemd affinity when the container
+     should really move to the container space cpus.
+
+     The sched_setaffinity call will always return an error (EINVAL or ENODEV)
+     when used like this. This is expected and part of the backward compatibility.
+
+     See: https://issues.redhat.com/browse/OCPBUGS-15102   */
+  ret = sched_setaffinity (args->pid, 0, NULL);
+  if (LIKELY (ret < 0))
+    {
+      if (UNLIKELY (! ((errno == EINVAL) || (errno == ENODEV))))
+        return crun_make_error (err, errno, "failed to reset affinity");
+    }
 
 success:
   *out = status;
@@ -393,7 +436,7 @@ libcrun_cgroup_enter_finalize (struct libcrun_cgroup_args *args, struct libcrun_
   current_cgroup += 3;
   to = strchr (current_cgroup, '\n');
   if (UNLIKELY (to == NULL))
-    return crun_make_error (err, 0, "cannot parse /proc/self/cgroup");
+    return crun_make_error (err, 0, "cannot parse `%s`", PROC_SELF_CGROUP);
   *to = '\0';
 
   ret = append_paths (&target_cgroup, err, current_cgroup, delegate_cgroup, NULL);
@@ -483,7 +526,7 @@ libcrun_cgroup_has_oom (struct libcrun_cgroup_status *status, libcrun_error_t *e
       }
 
     default:
-      return crun_make_error (err, 0, "invalid cgroup mode %d", cgroup_mode);
+      return crun_make_error (err, 0, "invalid cgroup mode `%d`", cgroup_mode);
     }
 
   it = content;
