@@ -274,12 +274,12 @@ restore_cgroup_v1_mount (runtime_spec_schema_config_schema *def, libcrun_error_t
   if (! has_cgroup_mount)
     return 0;
 
-  ret = read_all_file ("/proc/self/cgroup", &content, NULL, err);
+  ret = read_all_file (PROC_SELF_CGROUP, &content, NULL, err);
   if (UNLIKELY (ret < 0))
     return ret;
 
   if (UNLIKELY (content == NULL || content[0] == '\0'))
-    return crun_make_error (err, 0, "invalid content from /proc/self/cgroup");
+    return crun_make_error (err, 0, "invalid content from `%s`", PROC_SELF_CGROUP);
 
   for (from = strtok_r (content, "\n", &saveptr); from; from = strtok_r (NULL, "\n", &saveptr))
     {
@@ -342,12 +342,12 @@ checkpoint_cgroup_v1_mount (runtime_spec_schema_config_schema *def, libcrun_erro
   if (! has_cgroup_mount)
     return 0;
 
-  ret = read_all_file ("/proc/self/cgroup", &content, NULL, err);
+  ret = read_all_file (PROC_SELF_CGROUP, &content, NULL, err);
   if (UNLIKELY (ret < 0))
     return ret;
 
   if (UNLIKELY (content == NULL || content[0] == '\0'))
-    return crun_make_error (err, 0, "invalid content from /proc/self/cgroup");
+    return crun_make_error (err, 0, "invalid content from `%s`", PROC_SELF_CGROUP);
 
   for (from = strtok_r (content, "\n", &saveptr); from; from = strtok_r (NULL, "\n", &saveptr))
     {
@@ -433,11 +433,11 @@ libcrun_container_checkpoint_linux_criu (libcrun_container_status_t *status, lib
 
   ret = mkdir (cr_options->image_path, 0700);
   if (UNLIKELY ((ret == -1) && (errno != EEXIST)))
-    return crun_make_error (err, errno, "error creating checkpoint directory %s", cr_options->image_path);
+    return crun_make_error (err, errno, "error creating checkpoint directory `%s`", cr_options->image_path);
 
-  image_fd = open (cr_options->image_path, O_DIRECTORY);
+  image_fd = open (cr_options->image_path, O_DIRECTORY | O_CLOEXEC);
   if (UNLIKELY (image_fd == -1))
-    return crun_make_error (err, errno, "error opening checkpoint directory %s", cr_options->image_path);
+    return crun_make_error (err, errno, "error opening checkpoint directory `%s`", cr_options->image_path);
 
   libcriu_wrapper->criu_set_images_dir_fd (image_fd);
 
@@ -455,9 +455,9 @@ libcrun_container_checkpoint_linux_criu (libcrun_container_status_t *status, lib
    * crun to set it if the user has not selected a specific directory. */
   if (cr_options->work_path != NULL)
     {
-      work_fd = open (cr_options->work_path, O_DIRECTORY);
+      work_fd = open (cr_options->work_path, O_DIRECTORY | O_CLOEXEC);
       if (UNLIKELY (work_fd == -1))
-        return crun_make_error (err, errno, "error opening CRIU work directory %s", cr_options->work_path);
+        return crun_make_error (err, errno, "error opening CRIU work directory `%s`", cr_options->work_path);
 
       libcriu_wrapper->criu_set_work_dir_fd (work_fd);
     }
@@ -522,7 +522,7 @@ libcrun_container_checkpoint_linux_criu (libcrun_container_status_t *status, lib
 
   ret = libcriu_wrapper->criu_set_root (path);
   if (UNLIKELY (ret != 0))
-    return crun_make_error (err, 0, "error setting CRIU root to %s", path);
+    return crun_make_error (err, 0, "error setting CRIU root to `%s`", path);
 
   cgroup_mode = libcrun_get_cgroup_mode (err);
   if (UNLIKELY (cgroup_mode < 0))
@@ -624,8 +624,8 @@ libcrun_container_checkpoint_linux_criu (libcrun_container_status_t *status, lib
     }
 
   ret = libcriu_wrapper->criu_set_freeze_cgroup (freezer_path);
-  if (UNLIKELY (ret != 0))
-    return crun_make_error (err, ret, "CRIU: failed setting freezer %d", ret);
+  if (UNLIKELY (ret < 0))
+    return crun_make_error (err, -ret, "CRIU: failed setting freezer %d", ret);
 
   /* Set boolean options . */
   libcriu_wrapper->criu_set_leave_running (cr_options->leave_running);
@@ -702,7 +702,7 @@ prepare_restore_mounts (runtime_spec_schema_config_schema *def, char *root, libc
 
       root_fd = open (root, O_RDONLY | O_CLOEXEC);
       if (UNLIKELY (root_fd == -1))
-        return crun_make_error (err, errno, "error opening container root directory %s", root);
+        return crun_make_error (err, errno, "error opening container root directory `%s`", root);
 
       if (is_dir)
         {
@@ -758,9 +758,9 @@ libcrun_container_restore_linux_criu (libcrun_container_status_t *status, libcru
   if (UNLIKELY (cr_options->image_path == NULL))
     return crun_make_error (err, 0, "image path not set");
 
-  image_fd = open (cr_options->image_path, O_DIRECTORY);
+  image_fd = open (cr_options->image_path, O_DIRECTORY | O_CLOEXEC);
   if (UNLIKELY (image_fd == -1))
-    return crun_make_error (err, errno, "error opening checkpoint directory %s", cr_options->image_path);
+    return crun_make_error (err, errno, "error opening checkpoint directory `%s`", cr_options->image_path);
 
   libcriu_wrapper->criu_set_images_dir_fd (image_fd);
 
@@ -787,7 +787,7 @@ libcrun_container_restore_linux_criu (libcrun_container_status_t *status, libcru
      * and stderr being correctly redirected. */
     tree = yajl_tree_parse (buffer, err_buffer, sizeof (err_buffer));
     if (UNLIKELY (tree == NULL))
-      return crun_make_error (err, 0, "cannot parse descriptors file %s", DESCRIPTORS_FILENAME);
+      return crun_make_error (err, 0, "cannot parse descriptors file `%s`", DESCRIPTORS_FILENAME);
 
     if (tree && YAJL_IS_ARRAY (tree))
       {
@@ -814,9 +814,9 @@ libcrun_container_restore_linux_criu (libcrun_container_status_t *status, libcru
    * crun to set it if the user has not selected a specific directory. */
   if (cr_options->work_path != NULL)
     {
-      work_fd = open (cr_options->work_path, O_DIRECTORY);
+      work_fd = open (cr_options->work_path, O_DIRECTORY | O_CLOEXEC);
       if (UNLIKELY (work_fd == -1))
-        return crun_make_error (err, errno, "error opening CRIU work directory %s", cr_options->work_path);
+        return crun_make_error (err, errno, "error opening CRIU work directory `%s`", cr_options->work_path);
 
       libcriu_wrapper->criu_set_work_dir_fd (work_fd);
     }
@@ -861,12 +861,12 @@ libcrun_container_restore_linux_criu (libcrun_container_status_t *status, libcru
 
   ret = mkdir (root, 0755);
   if (UNLIKELY (ret == -1))
-    return crun_make_error (err, errno, "error creating restore directory %s", root);
+    return crun_make_error (err, errno, "error creating restore directory `%s`", root);
 
   ret = mount (status->rootfs, root, NULL, MS_BIND | MS_REC, NULL);
   if (UNLIKELY (ret == -1))
     {
-      ret = crun_make_error (err, errno, "error mounting restore directory %s", root);
+      ret = crun_make_error (err, errno, "error mounting restore directory `%s`", root);
       goto out;
     }
 
@@ -884,7 +884,7 @@ libcrun_container_restore_linux_criu (libcrun_container_status_t *status, libcru
   ret = libcriu_wrapper->criu_set_root (root);
   if (UNLIKELY (ret != 0))
     {
-      ret = crun_make_error (err, 0, "error setting CRIU root to %s", root);
+      ret = crun_make_error (err, 0, "error setting CRIU root to `%s`", root);
       goto out_umount;
     }
 
@@ -896,13 +896,14 @@ libcrun_container_restore_linux_criu (libcrun_container_status_t *status, libcru
    * The <key> needs to be the same as during checkpointing (extRootNetNS). */
   for (i = 0; i < def->linux->namespaces_len; i++)
     {
+      const int open_flags_for_inherit = O_RDONLY; /* Cannot be O_CLOEXEC as it is passed to the child process. */
       int value = libcrun_find_namespace (def->linux->namespaces[i]->type);
       if (UNLIKELY (value < 0))
         return crun_make_error (err, 0, "invalid namespace type: `%s`", def->linux->namespaces[i]->type);
 
       if (value == CLONE_NEWNET && def->linux->namespaces[i]->path != NULL)
         {
-          inherit_new_net_fd = open (def->linux->namespaces[i]->path, O_RDONLY);
+          inherit_new_net_fd = open (def->linux->namespaces[i]->path, open_flags_for_inherit);
           if (UNLIKELY (inherit_new_net_fd < 0))
             return crun_make_error (err, errno, "unable to open(): `%s`", def->linux->namespaces[i]->path);
 
@@ -911,7 +912,7 @@ libcrun_container_restore_linux_criu (libcrun_container_status_t *status, libcru
 
       if (value == CLONE_NEWPID && def->linux->namespaces[i]->path != NULL)
         {
-          inherit_new_pid_fd = open (def->linux->namespaces[i]->path, O_RDONLY);
+          inherit_new_pid_fd = open (def->linux->namespaces[i]->path, open_flags_for_inherit);
           if (UNLIKELY (inherit_new_pid_fd < 0))
             return crun_make_error (err, errno, "unable to open(): `%s`", def->linux->namespaces[i]->path);
 
@@ -972,7 +973,7 @@ libcrun_container_restore_linux_criu (libcrun_container_status_t *status, libcru
   if (UNLIKELY (ret <= 0))
     {
       ret = crun_make_error (err, 0,
-                             "CRIU restoring failed %d.  Please check CRIU logfile %s/%s",
+                             "CRIU restoring failed %d.  Please check CRIU logfile `%s/%s`",
                              ret, cr_options->work_path, CRIU_RESTORE_LOG_FILE);
       goto out_umount;
     }
@@ -981,22 +982,21 @@ libcrun_container_restore_linux_criu (libcrun_container_status_t *status, libcru
    * be necessary later when moving the process into its cgroup. */
   status->pid = ret;
 
-  if (LIKELY (ret > 0))
-    ret = libcrun_save_external_descriptors (container, ret, err);
+  ret = libcrun_save_external_descriptors (container, ret, err);
 
 out_umount:
   ret_out = umount (root);
   if (UNLIKELY (ret_out == -1))
     {
       rmdir (root);
-      return crun_make_error (err, errno, "error unmounting restore directory %s", root);
+      return crun_make_error (err, errno, "error unmounting restore directory `%s`", root);
     }
 out:
   ret_out = rmdir (root);
   if (UNLIKELY (ret == -1))
     return ret;
   if (UNLIKELY (ret_out == -1))
-    return crun_make_error (err, errno, "error removing restore directory %s", root);
+    return crun_make_error (err, errno, "error removing restore directory `%s`", root);
   return ret;
 }
 #endif
