@@ -40,20 +40,20 @@
 
 #if HAVE_DLOPEN && HAVE_WASMEDGE
 static int
-libwasmedge_load (void **cookie, libcrun_error_t *err arg_unused)
+libwasmedge_load (void **cookie, libcrun_error_t *err)
 {
   void *handle;
 
   handle = dlopen ("libwasmedge.so.0", RTLD_NOW);
   if (handle == NULL)
-    return crun_make_error (err, 0, "could not load `libwasmedge.so.0`: %s", dlerror ());
+    return crun_make_error (err, 0, "could not load `libwasmedge.so.0`: `%s`", dlerror ());
   *cookie = handle;
 
   return 0;
 }
 
 static int
-libwasmedge_unload (void *cookie, libcrun_error_t *err arg_unused)
+libwasmedge_unload (void *cookie, libcrun_error_t *err)
 {
   int r;
 
@@ -61,7 +61,7 @@ libwasmedge_unload (void *cookie, libcrun_error_t *err arg_unused)
     {
       r = dlclose (cookie);
       if (UNLIKELY (r < 0))
-        return crun_make_error (err, 0, "could not unload handle: %s", dlerror ());
+        return crun_make_error (err, 0, "could not unload handle: `%s`", dlerror ());
     }
   return 0;
 }
@@ -157,7 +157,7 @@ libwasmedge_exec (void *cookie, __attribute__ ((unused)) libcrun_container_t *co
 }
 
 static int
-wasmedge_can_handle_container (libcrun_container_t *container, libcrun_error_t *err arg_unused)
+wasmedge_can_handle_container (libcrun_container_t *container, libcrun_error_t *err)
 {
   return wasm_can_handle_container (container, err);
 }
