@@ -19,6 +19,7 @@
 #define _GNU_SOURCE
 
 #include <config.h>
+#include "blake3/blake3.h"
 #include "seccomp.h"
 #include "linux.h"
 #include "utils.h"
@@ -37,18 +38,11 @@
 #include <sys/stat.h>
 #include <sys/mman.h>
 
-#if HAVE_GCRYPT
-#  include <gcrypt.h>
-#endif
-
 #if HAVE_STDATOMIC_H
 #  include <stdatomic.h>
 #  ifndef HAVE_ATOMIC_INT
 #    define atomic_int volatile int
 #  endif
-#  ifndef HAVE_ATOMIC_BOOL
-#    define atomic_bool volatile int
-#  endif
 #endif
 
 #ifdef HAVE_SECCOMP
@@ -263,7 +257,7 @@ libcrun_apply_seccomp (int infd, int listener_receiver_fd, const char *receiver_
 #  endif
 
 #  ifdef HAVE_MEMFD_CREATE
-      memfd = memfd_create ("seccomp-helper-memfd", O_RDWR);
+      memfd = memfd_create ("seccomp-helper-memfd", O_RDWR | MFD_CLOEXEC);
       if (UNLIKELY (memfd < 0))
         return crun_make_error (err, errno, "memfd_create");
 #  else
@@ -372,55 +366,34 @@ seccomp_action_supports_errno (const char *action)
 
    Returns:
    < 0 in case of errors
-     0 if the checksum is not supported
-   > 0 the checksum is supported and the value is in OUT.
+   == 0 the checksum is supported and the value is in OUT.
  */
 static int
 calculate_seccomp_checksum (runtime_spec_schema_config_linux_seccomp *seccomp, unsigned int seccomp_gen_options, seccomp_checksum_t out, libcrun_error_t *err)
 {
-#if HAVE_GCRYPT
-  static atomic_bool initialized = false;
-  gcry_error_t gcrypt_err;
+  blake3_hasher hasher;
+  unsigned char hash[32];
   struct utsname utsbuf;
-  unsigned char *res;
-  gcry_md_hd_t hd;
   size_t i;
   int ret;
 
-  if (! initialized && ! gcry_control (GCRYCTL_INITIALIZATION_FINISHED_P))
-    {
-      const char *needed_version = "1.0.0";
-      if (! gcry_check_version (needed_version))
-        {
-          return libcrun_make_error (err, 0, "libgcrypt is too old (need `%s`, have `%s`)",
-                                     needed_version, gcry_check_version (NULL));
-        }
-      gcry_control (GCRYCTL_DISABLE_SECMEM, 0);
-      gcry_control (GCRYCTL_INITIALIZATION_FINISHED, 0);
-      initialized = true;
-    }
+  blake3_hasher_init (&hasher);
 
-#  define PROCESS_STRING(X)                      \
-    do                                           \
-      {                                          \
-        if (X)                                   \
-          {                                      \
-            gcry_md_write (hd, (X), strlen (X)); \
-          }                                      \
-    } while (0)
-#  define PROCESS_DATA(X)                     \
-    do                                        \
-      {                                       \
-        gcry_md_write (hd, &(X), sizeof (X)); \
-    } while (0)
-
-  gcrypt_err = gcry_md_open (&hd, GCRY_MD_SHA256, 0);
-  if (gcrypt_err)
-    return crun_make_error (err, EINVAL, "internal libgcrypt error: %s", gcry_strerror (gcrypt_err));
+#define PROCESS_STRING(X)                                  \
+  do                                                       \
+    {                                                      \
+      if (X)                                               \
+        blake3_hasher_update (&hasher, (X), strlen ((X))); \
+  } while (0)
+#define PROCESS_DATA(X)                                   \
+  do                                                      \
+    {                                                     \
+      blake3_hasher_update (&hasher, &(X), sizeof ((X))); \
+  } while (0)
 
   PROCESS_STRING (PACKAGE_VERSION);
 
-#  ifdef HAVE_SECCOMP
+#ifdef HAVE_SECCOMP
   {
     const struct scmp_version *version = seccomp_version ();
 
@@ -428,7 +401,7 @@ calculate_seccomp_checksum (runtime_spec_schema_config_linux_seccomp *seccomp, u
     PROCESS_DATA (version->minor);
     PROCESS_DATA (version->micro);
   }
-#  endif
+#endif
 
   memset (&utsbuf, 0, sizeof (utsbuf));
   ret = uname (&utsbuf);
@@ -441,6 +414,7 @@ calculate_seccomp_checksum (runtime_spec_schema_config_linux_seccomp *seccomp, u
 
   PROCESS_DATA (seccomp_gen_options);
 
+  PROCESS_DATA (seccomp->default_errno_ret);
   PROCESS_STRING (seccomp->default_action);
   for (i = 0; i < seccomp->flags_len; i++)
     PROCESS_STRING (seccomp->flags[i]);
@@ -466,24 +440,15 @@ calculate_seccomp_checksum (runtime_spec_schema_config_linux_seccomp *seccomp, u
         }
     }
 
-  res = gcry_md_read (hd, GCRY_MD_SHA256);
+  blake3_hasher_finalize (&hasher, hash, sizeof (hash));
+
   for (i = 0; i < 32; i++)
-    sprintf (&out[i * 2], "%02x", res[i]);
+    sprintf (&out[i * 2], "%02x", hash[i]);
   out[64] = 0;
 
-  gcry_md_close (hd);
-
-#  undef PROCESS_STRING
-#  undef PROCESS_DATA
+#undef PROCESS_STRING
+#undef PROCESS_DATA
   return 1;
-#else
-  (void) seccomp;
-  (void) seccomp_gen_options;
-  (void) out;
-  (void) err;
-  out[0] = 0;
-  return 0;
-#endif
 }
 
 static int
@@ -671,7 +636,7 @@ find_in_cache (struct libcrun_seccomp_gen_ctx_s *ctx, int dirfd, const char *des
 
   /* if the checksum could not be computed, returns early.  */
   ret = calculate_seccomp_checksum (seccomp, ctx->options, ctx->checksum, err);
-  if (UNLIKELY (ret <= 0))
+  if (UNLIKELY (ret < 0))
     return ret;
 
   ret = append_paths (&cache_file_path, err, SECCOMP_CACHE_DIR, ctx->checksum, NULL);
@@ -926,7 +891,7 @@ libcrun_open_seccomp_bpf (struct libcrun_seccomp_gen_ctx_s *ctx, int *fd, libcru
           goto open_existing_file;
         }
 
-      ret = TEMP_FAILURE_RETRY (openat (dirfd, dest_path, O_RDWR | O_CREAT, 0700));
+      ret = TEMP_FAILURE_RETRY (openat (dirfd, dest_path, O_CLOEXEC | O_RDWR | O_CREAT, 0700));
       if (UNLIKELY (ret < 0))
         return crun_make_error (err, errno, "open `seccomp.bpf`");
       ctx->fd = *fd = ret;
@@ -934,7 +899,7 @@ libcrun_open_seccomp_bpf (struct libcrun_seccomp_gen_ctx_s *ctx, int *fd, libcru
   else
     {
     open_existing_file:
-      ret = TEMP_FAILURE_RETRY (openat (dirfd, dest_path, O_RDONLY));
+      ret = TEMP_FAILURE_RETRY (openat (dirfd, dest_path, O_CLOEXEC | O_RDONLY));
       if (UNLIKELY (ret < 0))
         {
           if (errno == ENOENT)
